@@ -1,35 +1,38 @@
-from dask.base import tokenize
-import numpy as np
-import dask.array as da
-import xarray
-from xarray import DataArray
 from typing import (
     Any,
+    Callable,
+    Dict,
     Hashable,
     List,
     Literal,
     Optional,
+    Sequence,
     Tuple,
     Union,
-    Sequence,
-    Callable,
-    Dict,
     cast,
 )
-from dask.utils import apply
-from dask.core import flatten
-from dask.array.routines import aligned_coarsen_chunks
-from dask.highlevelgraph import HighLevelGraph
+
+import dask.array as da
+import numpy as np
+import xarray
 from dask.array import Array
+from dask.array.routines import aligned_coarsen_chunks
+from dask.base import tokenize
+from dask.core import flatten
+from dask.highlevelgraph import HighLevelGraph
+from dask.utils import apply
 from numpy.typing import NDArray
+from xarray import DataArray
 from xarray.core.utils import is_dict_like
 
+from xarray_multiscale.reducers import WindowedReducer
+
 CHUNK_MODES = ("rechunk", "minimum")
 
 
 def multiscale(
-    array: Any,
-    reduction: Callable[[NDArray[Any], Tuple[int, ...], Dict[Any, Any]], NDArray[Any]],
+    array: NDArray[Any],
+    reduction: WindowedReducer,
     scale_factors: Union[Sequence[int], int],
     depth: int = -1,
     pad_mode: str = "crop",
@@ -43,12 +46,12 @@ def multiscale(
 
     Parameters
     ----------
-    array : numpy array, dask array, or xarray DataArray
-        The array to be downscaled
+    array : Array-like, e.g. Numpy array, Dask array
+        The array to be downscaled.
 
     reduction : callable
         A function that aggregates chunks of data over windows.
-        See the documentation of `dask.array.coarsen` for the expected
+        See `` for the expected
         signature of this callable.
 
     scale_factors : int or sequence of ints
@@ -63,16 +66,20 @@ def multiscale(
         depth + 1 downscaled arrays.
 
     pad_mode : string or None, default=None
-        How arrays should be padded prior to downscaling in order to ensure that each array dimension
-        is evenly divisible by the respective scale factor. When set to `None` (default), the input will be sliced before downscaling
-        if its dimensions are not divisible by `scale_factors`.
+        How arrays should be padded prior to downscaling in order to ensure
+        that each array dimension is evenly divisible by the respective
+        scale factor. When set to `None` (default), the input will be
+        cropped before downscaling if its dimensions are not divisible by
+        ``scale_factors``.
 
     preserve_dtype : bool, default=True
-        Determines whether the multiresolution arrays are all cast to the same dtype as the input.
+        Determines whether the multiresolution arrays are all explicitly
+        cast to the same dtype as the input.
 
     chunks : sequence or dict of ints, or None, default=None
-        If `chunks` is supplied, all output arrays are returned with this chunking. If not None, this
-        argument is passed directly to the `xarray.DataArray.chunk` method of each output array.
+        If `chunks` is supplied, all output arrays are returned with this
+        chunking. If not None, this argument is passed directly to the
+        `xarray.DataArray.chunk` method of each output array.
 
     chunk_mode : str, default='rechunk'
         `chunk_mode` determines how to interpret the `chunks` keyword argument.
@@ -82,21 +89,44 @@ def multiscale(
         only if that array has a chunk size smaller than `chunks`.
 
     chained : bool, default=True
-        If True (default), the nth downscaled array is generated by applying the reduction function on the n-1th
-        downscaled array with the user-supplied `scale_factors`. This means that the nth downscaled array directly depends on the n-1th
-        downscaled array. Note that nonlinear reductions like the windowed mode may give inaccurate results with `chained` set to True.
+        If True (default), the nth downscaled array is generated by
+        applying the reduction function on the n-1th downscaled array with
+        the user-supplied `scale_factors`. This means that the nth
+        downscaled array directly depends on the n-1th downscaled array.
+        Note that nonlinear reductions like the windowed mode may give
+        inaccurate results with `chained` set to True.
 
-        If False, the nth downscaled array is generated by applying the reduction function on the 0th downscaled array
-        (i.e., the input array) with the `scale_factors` raised to the nth power. This means that the nth downscaled array directly
-        depends on the input array.
+        If False, the nth downscaled array is generated by applying the
+        reduction function on the 0th downscaled array
+        (i.e., the input array) with the `scale_factors` raised to the nth
+        power. This means that the nth downscaled array directly depends
+        on the input array.
 
     Returns
     -------
     result : list of DataArrays
-        The `coords` attribute of these DataArrays properties that track the changing offset (if any)
-        induced by the downsampling operation. Additionally, the scale factors are stored each DataArray's attrs propery under the key `scale_factors`
+        The `coords` attribute of these DataArrays track the changing
+        offset (if any) induced by the downsampling operation.
+        Additionally, the scale factors are stored each DataArray's
+        attrs propery under the key `scale_factors`
 
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from xarray_multiscale import multiscale
+    >>> from xarray_multiscale.reducers import windowed_mean
+    >>> multiscale(np.arange(4), windowed_mean, 2)
+    [<xarray.DataArray 'array-3fddd342b603c7121f36e43be77be0cf' (dim_0: 4)>
+    dask.array<array, shape=(4,), dtype=int64, chunksize=(4,), chunktype=numpy.ndarray>
+    Coordinates:
+    * dim_0    (dim_0) float64 0.0 1.0 2.0 3.0, <xarray.DataArray 'downscale-f8b482b55696e22c132f3c9b8d583090' (dim_0: 2)>
+    dask.array<astype, shape=(2,), dtype=int64, chunksize=(2,), chunktype=numpy.ndarray>
+    Coordinates:
+    * dim_0    (dim_0) float64 0.5 2.5, <xarray.DataArray 'downscale-8b8e1406654a7ef2394a47a0f32054db' (dim_0: 1)>
+    dask.array<astype, shape=(1,), dtype=int64, chunksize=(1,), chunktype=numpy.ndarray>
+    Coordinates:
+    * dim_0    (dim_0) float64 1.5]
     """
     if chunk_mode not in CHUNK_MODES:
         raise ValueError(f"chunk_mode must be one of {CHUNK_MODES}, not {chunk_mode}")
@@ -213,7 +243,8 @@ def adjust_shape(
     array: DataArray, scale_factors: Sequence[int], mode: Union[str, None]
 ) -> DataArray:
     """
-    Pad or crop array such that its new dimensions are evenly divisible by a set of integers.
+    Pad or crop array such that its new dimensions are evenly
+    divisible by a set of integers.
 
     Parameters
     ----------
@@ -221,14 +252,16 @@ def adjust_shape(
         Array that will be padded.
 
     scale_factors : Sequence of ints
-        The output array is guaranteed to have dimensions that are each evenly divisible
-        by the corresponding scale factor, and chunks that are smaller than or equal
-        to the scale factor (if the array has chunks)
+        The output array is guaranteed to have dimensions that are each
+        evenly divisible by the corresponding scale factor, and chunks
+        that are smaller than or equal to the scale factor
+        (if the array has chunks)
 
     mode : str
-        If set to "crop", then the input array will be cropped as needed. Otherwise,
-        this is the edge mode used by the padding routine. This parameter will be passed to
-        `dask.array.pad` as the `mode` keyword.
+        If set to "crop", then the input array will be cropped as needed.
+        Otherwise, this is the edge mode used by the padding routine.
+        This parameter will be passed to `dask.array.pad` as the `mode`
+        keyword.
 
     Returns
     -------
@@ -297,7 +330,9 @@ def downscale(
         to_downscale.data, reduction, scale_factors, **kwargs
     )
     downscaled_coords = downscale_coords(to_downscale, scale_factors)
-    return DataArray(downscaled_data, downscaled_coords, attrs=array.attrs)
+    return DataArray(
+        downscaled_data, downscaled_coords, attrs=array.attrs, dims=array.dims
+    )
 
 
 def downscale_coords(
@@ -333,7 +368,7 @@ def get_downscale_depth(
 
     _scale_factors = np.array(scale_factors).astype("int")
     _shape = np.array(shape).astype("int")
-    valid = (_scale_factors > 1)
+    valid = _scale_factors > 1
     if not valid.any():
         result = 0
     else:
@@ -347,9 +382,11 @@ def get_downscale_depth(
 
 def get_downsampled_offset(scale_factors: Sequence[int]) -> Any:
     """
-    For a given number of dimensions and a sequence of downscale factors, calculate the starting offset of the downscaled
-    array in the units of the full-resolution data.
+    For a given number of dimensions and a sequence of downscale factors,
+    calculate the starting offset of the downscaled array in the units of
+    the full-resolution data.
     """
+
     return np.array([np.arange(s).mean() for s in scale_factors])
 
 
